=== modified file 'cmd/juju/addmachine.go'
--- cmd/juju/addmachine.go	2013-07-16 00:47:26 +0000
+++ cmd/juju/addmachine.go	2013-07-29 15:15:41 +0000
@@ -5,14 +5,17 @@
 
 import (
 	"fmt"
+	"strings"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/juju"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
-	"strings"
 )
 
 // AddMachineCommand starts a new machine and registers it in the environment.
@@ -54,7 +57,7 @@
 	}
 	// container arg can either be 'type:machine' or 'type'
 	if c.ContainerType, err = instance.ParseSupportedContainerType(containerSpec); err != nil {
-		if !state.IsMachineOrNewContainer(containerSpec) {
+		if !names.IsMachineOrNewContainer(containerSpec) {
 			return fmt.Errorf("malformed container argument %q", containerSpec)
 		}
 		sep := strings.Index(containerSpec, ":")

=== modified file 'cmd/juju/addunit.go'
--- cmd/juju/addunit.go	2013-07-18 22:57:04 +0000
+++ cmd/juju/addunit.go	2013-07-29 15:15:41 +0000
@@ -6,10 +6,12 @@
 import (
 	"errors"
 	"fmt"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/statecmd"
 )
@@ -34,7 +36,7 @@
 		if c.NumUnits > 1 {
 			return errors.New("cannot use --num-units > 1 with --to")
 		}
-		if !state.IsMachineOrNewContainer(c.ToMachineSpec) {
+		if !names.IsMachineOrNewContainer(c.ToMachineSpec) {
 			return fmt.Errorf("invalid --to parameter %q", c.ToMachineSpec)
 		}
 	}

=== modified file 'cmd/juju/constraints.go'
--- cmd/juju/constraints.go	2013-05-02 15:55:42 +0000
+++ cmd/juju/constraints.go	2013-07-29 15:15:41 +0000
@@ -5,11 +5,13 @@
 
 import (
 	"fmt"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/statecmd"
 )
@@ -44,7 +46,7 @@
 
 func (c *GetConstraintsCommand) Init(args []string) error {
 	if len(args) > 0 {
-		if !state.IsServiceName(args[0]) {
+		if !names.IsServiceName(args[0]) {
 			return fmt.Errorf("invalid service name %q", args[0])
 		}
 		c.ServiceName, args = args[0], args[1:]
@@ -98,7 +100,7 @@
 }
 
 func (c *SetConstraintsCommand) Init(args []string) (err error) {
-	if c.ServiceName != "" && !state.IsServiceName(c.ServiceName) {
+	if c.ServiceName != "" && !names.IsServiceName(c.ServiceName) {
 		return fmt.Errorf("invalid service name %q", c.ServiceName)
 	}
 	c.Constraints, err = constraints.Parse(args...)

=== modified file 'cmd/juju/deploy.go'
--- cmd/juju/deploy.go	2013-07-18 01:19:32 +0000
+++ cmd/juju/deploy.go	2013-07-29 15:15:41 +0000
@@ -6,13 +6,15 @@
 import (
 	"errors"
 	"fmt"
+	"os"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
-	"os"
+	"launchpad.net/juju-core/names"
 )
 
 type DeployCommand struct {
@@ -76,7 +78,7 @@
 func (c *DeployCommand) Init(args []string) error {
 	switch len(args) {
 	case 2:
-		if !state.IsServiceName(args[1]) {
+		if !names.IsServiceName(args[1]) {
 			return fmt.Errorf("invalid service name %q", args[1])
 		}
 		c.ServiceName = args[1]

=== modified file 'cmd/juju/destroymachine.go'
--- cmd/juju/destroymachine.go	2013-05-02 15:55:42 +0000
+++ cmd/juju/destroymachine.go	2013-07-29 15:15:41 +0000
@@ -5,9 +5,10 @@
 
 import (
 	"fmt"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 )
 
 // DestroyMachineCommand causes an existing machine to be destroyed.
@@ -31,7 +32,7 @@
 		return fmt.Errorf("no machines specified")
 	}
 	for _, id := range args {
-		if !state.IsMachineId(id) {
+		if !names.IsMachineId(id) {
 			return fmt.Errorf("invalid machine id %q", id)
 		}
 	}

=== modified file 'cmd/juju/destroyservice.go'
--- cmd/juju/destroyservice.go	2013-05-02 15:55:42 +0000
+++ cmd/juju/destroyservice.go	2013-07-29 15:15:41 +0000
@@ -5,9 +5,10 @@
 
 import (
 	"fmt"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/statecmd"
 )
@@ -31,7 +32,7 @@
 	if len(args) == 0 {
 		return fmt.Errorf("no service specified")
 	}
-	if !state.IsServiceName(args[0]) {
+	if !names.IsServiceName(args[0]) {
 		return fmt.Errorf("invalid service name %q", args[0])
 	}
 	c.ServiceName, args = args[0], args[1:]

=== modified file 'cmd/juju/destroyunit.go'
--- cmd/juju/destroyunit.go	2013-05-02 15:55:42 +0000
+++ cmd/juju/destroyunit.go	2013-07-29 15:15:41 +0000
@@ -6,9 +6,10 @@
 import (
 	"errors"
 	"fmt"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/statecmd"
 )
@@ -34,7 +35,7 @@
 		return errors.New("no units specified")
 	}
 	for _, name := range c.UnitNames {
-		if !state.IsUnitName(name) {
+		if !names.IsUnitName(name) {
 			return fmt.Errorf("invalid unit name %q", name)
 		}
 	}

=== modified file 'cmd/juju/resolved.go'
--- cmd/juju/resolved.go	2013-05-02 15:55:42 +0000
+++ cmd/juju/resolved.go	2013-07-29 15:15:41 +0000
@@ -5,10 +5,12 @@
 
 import (
 	"fmt"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 )
 
 // ResolvedCommand marks a unit in an error state as ready to continue.
@@ -35,7 +37,7 @@
 func (c *ResolvedCommand) Init(args []string) error {
 	if len(args) > 0 {
 		c.UnitName = args[0]
-		if !state.IsUnitName(c.UnitName) {
+		if !names.IsUnitName(c.UnitName) {
 			return fmt.Errorf("invalid unit name %q", c.UnitName)
 		}
 		args = args[1:]

=== modified file 'cmd/juju/ssh.go'
--- cmd/juju/ssh.go	2013-06-28 12:26:09 +0000
+++ cmd/juju/ssh.go	2013-07-29 15:15:41 +0000
@@ -6,12 +6,14 @@
 import (
 	"errors"
 	"fmt"
+	"os/exec"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/juju"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
-	"os/exec"
 )
 
 // SSHCommand is responsible for launching a ssh shell on a given unit or machine.
@@ -75,13 +77,13 @@
 
 func (c *SSHCommon) hostFromTarget(target string) (string, error) {
 	// is the target the id of a machine ?
-	if state.IsMachineId(target) {
+	if names.IsMachineId(target) {
 		log.Infof("looking up address for machine %s...", target)
 		// TODO(dfc) maybe we should have machine.PublicAddress() ?
 		return c.machinePublicAddress(target)
 	}
 	// maybe the target is a unit ?
-	if state.IsUnitName(target) {
+	if names.IsUnitName(target) {
 		log.Infof("looking up address for unit %q...", c.Target)
 		unit, err := c.State.Unit(target)
 		if err != nil {

=== modified file 'cmd/juju/upgradecharm.go'
--- cmd/juju/upgradecharm.go	2013-06-04 15:34:30 +0000
+++ cmd/juju/upgradecharm.go	2013-07-29 15:15:41 +0000
@@ -6,12 +6,14 @@
 import (
 	"errors"
 	"fmt"
+	"os"
+
 	"launchpad.net/gnuflag"
+
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
-	"os"
+	"launchpad.net/juju-core/names"
 )
 
 // UpgradeCharm is responsible for upgrading a service's charm.
@@ -86,7 +88,7 @@
 func (c *UpgradeCharmCommand) Init(args []string) error {
 	switch len(args) {
 	case 1:
-		if !state.IsServiceName(args[0]) {
+		if !names.IsServiceName(args[0]) {
 			return fmt.Errorf("invalid service name %q", args[0])
 		}
 		c.ServiceName = args[0]

=== modified file 'cmd/jujud/machine.go'
--- cmd/jujud/machine.go	2013-07-26 15:16:59 +0000
+++ cmd/jujud/machine.go	2013-07-29 15:15:41 +0000
@@ -18,6 +18,7 @@
 	"launchpad.net/juju-core/environs/provider"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/state/api/machineagent"
@@ -59,7 +60,7 @@
 
 // Init initializes the command for running.
 func (a *MachineAgent) Init(args []string) error {
-	if !state.IsMachineId(a.MachineId) {
+	if !names.IsMachineId(a.MachineId) {
 		return fmt.Errorf("--machine-id option must be set, and expects a non-negative integer")
 	}
 	if err := a.Conf.checkArgs(args); err != nil {
@@ -304,7 +305,7 @@
 }
 
 func (a *MachineAgent) Tag() string {
-	return state.MachineTag(a.MachineId)
+	return names.MachineTag(a.MachineId)
 }
 
 // Below pieces are used for testing,to give us access to the *State opened

=== modified file 'cmd/jujud/machine_test.go'
--- cmd/jujud/machine_test.go	2013-07-25 11:03:16 +0000
+++ cmd/jujud/machine_test.go	2013-07-29 15:15:41 +0000
@@ -21,6 +21,7 @@
 	envtesting "launchpad.net/juju-core/environs/testing"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/state/api/params"
@@ -70,7 +71,7 @@
 	c.Assert(err, IsNil)
 	err = m.SetPassword("machine-password")
 	c.Assert(err, IsNil)
-	conf, tools := s.agentSuite.primeAgent(c, state.MachineTag(m.Id()), "machine-password")
+	conf, tools := s.agentSuite.primeAgent(c, names.MachineTag(m.Id()), "machine-password")
 	conf.MachineNonce = state.BootstrapNonce
 	conf.APIInfo.Nonce = conf.MachineNonce
 	err = conf.Write()

=== modified file 'cmd/jujud/unit.go'
--- cmd/jujud/unit.go	2013-07-29 10:32:03 +0000
+++ cmd/jujud/unit.go	2013-07-29 15:15:41 +0000
@@ -11,6 +11,7 @@
 	"launchpad.net/tomb"
 
 	"launchpad.net/juju-core/cmd"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/worker"
@@ -46,7 +47,7 @@
 	if a.UnitName == "" {
 		return requiredError("unit-name")
 	}
-	if !state.IsUnitName(a.UnitName) {
+	if !names.IsUnitName(a.UnitName) {
 		return fmt.Errorf(`--unit-name option expects "<service>/<n>" argument`)
 	}
 	if err := a.Conf.checkArgs(args); err != nil {
@@ -110,5 +111,5 @@
 }
 
 func (a *UnitAgent) Tag() string {
-	return state.UnitTag(a.UnitName)
+	return names.UnitTag(a.UnitName)
 }

=== modified file 'cmd/jujud/upgradevalidation.go'
--- cmd/jujud/upgradevalidation.go	2013-07-17 07:03:30 +0000
+++ cmd/jujud/upgradevalidation.go	2013-07-29 15:15:41 +0000
@@ -14,6 +14,7 @@
 	"launchpad.net/juju-core/container/lxc"
 	"launchpad.net/juju-core/environs/provider"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/utils"
@@ -60,7 +61,11 @@
 	//     package for exactly the same reasons.
 	// Later, post-precise LTS, when we have updated lxc, we can bring this
 	// back in to have nested lxc, but until then, we have to avoid it.
-	containerType := state.ContainerTypeFromId(state.MachineIdFromTag(machineTag))
+	id, err := names.MachineIdFromTag(machineTag)
+	if err != nil {
+		return err
+	}
+	containerType := state.ContainerTypeFromId(id)
 	providerType := os.Getenv("JUJU_PROVIDER_TYPE")
 	if providerType == provider.Local || containerType == instance.LXC {
 		return nil

=== modified file 'container/lxc/lxc.go'
--- container/lxc/lxc.go	2013-07-22 04:23:33 +0000
+++ container/lxc/lxc.go	2013-07-29 15:15:41 +0000
@@ -19,6 +19,7 @@
 	"launchpad.net/juju-core/environs/cloudinit"
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 )
@@ -81,7 +82,7 @@
 	stateInfo *state.Info,
 	apiInfo *api.Info) (instance.Instance, error) {
 
-	name := state.MachineTag(machineId)
+	name := names.MachineTag(machineId)
 	if manager.name != "" {
 		name = fmt.Sprintf("%s-%s", manager.name, name)
 	}

=== modified file 'environs/azure/customdata_test.go'
--- environs/azure/customdata_test.go	2013-07-22 04:19:25 +0000
+++ environs/azure/customdata_test.go	2013-07-29 15:15:41 +0000
@@ -11,6 +11,7 @@
 	"launchpad.net/juju-core/agent/tools"
 	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/environs/cloudinit"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/testing"
@@ -32,12 +33,12 @@
 		StateInfo: &state.Info{
 			CACert: []byte(testing.CACert),
 			Addrs:  []string{"127.0.0.1:123"},
-			Tag:    state.MachineTag(machineID),
+			Tag:    names.MachineTag(machineID),
 		},
 		APIInfo: &api.Info{
 			CACert: []byte(testing.CACert),
 			Addrs:  []string{"127.0.0.1:123"},
-			Tag:    state.MachineTag(machineID),
+			Tag:    names.MachineTag(machineID),
 		},
 		ProviderType: "azure",
 	}

=== modified file 'environs/cloudinit/cloudinit.go'
--- environs/cloudinit/cloudinit.go	2013-07-24 16:08:50 +0000
+++ environs/cloudinit/cloudinit.go	2013-07-29 15:15:41 +0000
@@ -17,6 +17,7 @@
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log/syslog"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/upstart"
@@ -167,7 +168,7 @@
 	// It would be cleaner to change bootstrap-state to
 	// be responsible for starting the machine agent itself,
 	// but this would not be backwardly compatible.
-	machineTag := state.MachineTag(cfg.MachineId)
+	machineTag := names.MachineTag(cfg.MachineId)
 	_, err := cfg.addAgentInfo(c, machineTag)
 	if err != nil {
 		return nil, err
@@ -216,10 +217,10 @@
 	var configRenderer syslog.SyslogConfigRenderer
 	if cfg.StateServer {
 		configRenderer = syslog.NewAccumulateConfig(
-			state.MachineTag(cfg.MachineId))
+			names.MachineTag(cfg.MachineId))
 	} else {
 		configRenderer = syslog.NewForwardConfig(
-			state.MachineTag(cfg.MachineId), cfg.stateHostAddrs())
+			names.MachineTag(cfg.MachineId), cfg.stateHostAddrs())
 	}
 	content, err := configRenderer.Render()
 	if err != nil {
@@ -373,7 +374,7 @@
 
 func verifyConfig(cfg *MachineConfig) (err error) {
 	defer utils.ErrorContextf(&err, "invalid machine configuration")
-	if !state.IsMachineId(cfg.MachineId) {
+	if !names.IsMachineId(cfg.MachineId) {
 		return fmt.Errorf("invalid machine id")
 	}
 	if cfg.DataDir == "" {
@@ -426,13 +427,13 @@
 		if len(cfg.StateInfo.Addrs) == 0 {
 			return fmt.Errorf("missing state hosts")
 		}
-		if cfg.StateInfo.Tag != state.MachineTag(cfg.MachineId) {
+		if cfg.StateInfo.Tag != names.MachineTag(cfg.MachineId) {
 			return fmt.Errorf("entity tag must match started machine")
 		}
 		if len(cfg.APIInfo.Addrs) == 0 {
 			return fmt.Errorf("missing API hosts")
 		}
-		if cfg.APIInfo.Tag != state.MachineTag(cfg.MachineId) {
+		if cfg.APIInfo.Tag != names.MachineTag(cfg.MachineId) {
 			return fmt.Errorf("entity tag must match started machine")
 		}
 	}

=== modified file 'environs/dummy/environs.go'
--- environs/dummy/environs.go	2013-07-19 01:18:26 +0000
+++ environs/dummy/environs.go	2013-07-29 15:15:41 +0000
@@ -25,24 +25,26 @@
 import (
 	"errors"
 	"fmt"
+	"net"
+	"net/http"
+	"os"
+	"strings"
+	"sync"
+	"time"
+
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/environs/config"
 	envtesting "launchpad.net/juju-core/environs/testing"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/schema"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/state/apiserver"
 	"launchpad.net/juju-core/testing"
 	"launchpad.net/juju-core/utils"
-	"net"
-	"net/http"
-	"os"
-	"strings"
-	"sync"
-	"time"
 )
 
 // stateInfo returns a *state.Info which allows clients to connect to the
@@ -566,10 +568,10 @@
 	if _, ok := e.Config().CACert(); !ok {
 		return nil, nil, fmt.Errorf("no CA certificate in environment configuration")
 	}
-	if info.Tag != state.MachineTag(machineId) {
+	if info.Tag != names.MachineTag(machineId) {
 		return nil, nil, fmt.Errorf("entity tag must match started machine")
 	}
-	if apiInfo.Tag != state.MachineTag(machineId) {
+	if apiInfo.Tag != names.MachineTag(machineId) {
 		return nil, nil, fmt.Errorf("entity tag must match started machine")
 	}
 	i := &dummyInstance{

=== modified file 'environs/local/environ.go'
--- environs/local/environ.go	2013-07-24 16:08:50 +0000
+++ environs/local/environ.go	2013-07-29 15:15:41 +0000
@@ -21,6 +21,7 @@
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/environs/localstorage"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/upstart"
@@ -474,7 +475,7 @@
 	err = tools.UnpackTools(dataDir, agentTools, toolsFile)
 
 	machineId := "0" // Always machine 0
-	tag := state.MachineTag(machineId)
+	tag := names.MachineTag(machineId)
 	toolsDir := tools.SharedToolsDir(dataDir, agentTools.Version)
 	logDir := env.config.logDir()
 	logConfig := "--debug" // TODO(thumper): specify loggo config
@@ -507,7 +508,7 @@
 		logger.Errorf("failed to get state info to write bootstrap agent file: %v", err)
 		return err
 	}
-	tag := state.MachineTag("0")
+	tag := names.MachineTag("0")
 	info.Tag = tag
 	apiInfo.Tag = tag
 	conf := &agent.Conf{

=== modified file 'environs/openstack/provider.go'
--- environs/openstack/provider.go	2013-07-24 13:07:02 +0000
+++ environs/openstack/provider.go	2013-07-29 15:15:41 +0000
@@ -9,11 +9,17 @@
 	"errors"
 	"fmt"
 	"io/ioutil"
+	"net/http"
+	"strings"
+	"sync"
+	"time"
+
 	"launchpad.net/goose/client"
 	gooseerrors "launchpad.net/goose/errors"
 	"launchpad.net/goose/identity"
 	"launchpad.net/goose/nova"
 	"launchpad.net/goose/swift"
+
 	"launchpad.net/juju-core/agent/tools"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
@@ -23,13 +29,10 @@
 	"launchpad.net/juju-core/environs/instances"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/utils"
-	"net/http"
-	"strings"
-	"sync"
-	"time"
 )
 
 type environProvider struct{}
@@ -871,7 +874,7 @@
 }
 
 func (e *environ) machineFullName(machineId string) string {
-	return fmt.Sprintf("juju-%s-%s", e.Name(), state.MachineTag(machineId))
+	return fmt.Sprintf("juju-%s-%s", e.Name(), names.MachineTag(machineId))
 }
 
 // machinesFilter returns a nova.Filter matching all machines in the environment.

=== modified file 'juju/conn.go'
--- juju/conn.go	2013-07-19 02:21:57 +0000
+++ juju/conn.go	2013-07-29 15:15:41 +0000
@@ -23,6 +23,7 @@
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/utils"
 )
@@ -338,7 +339,7 @@
 					mid = machineIdSpec
 				}
 			}
-			if !state.IsMachineId(mid) {
+			if !names.IsMachineId(mid) {
 				return nil, fmt.Errorf("invalid force machine id %q", mid)
 			}
 

=== modified file 'juju/testing/conn.go'
--- juju/testing/conn.go	2013-07-19 17:12:37 +0000
+++ juju/testing/conn.go	2013-07-29 15:15:41 +0000
@@ -6,7 +6,11 @@
 import (
 	"fmt"
 	"io/ioutil"
+	"os"
+	"path/filepath"
+
 	. "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
@@ -14,12 +18,11 @@
 	"launchpad.net/juju-core/environs/dummy"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/juju"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/testing"
 	"launchpad.net/juju-core/version"
-	"os"
-	"path/filepath"
 )
 
 // JujuConnSuite provides a freshly bootstrapped juju.Conn
@@ -60,7 +63,7 @@
 func FakeStateInfo(machineId string) *state.Info {
 	return &state.Info{
 		Addrs:    []string{"0.1.2.3:1234"},
-		Tag:      state.MachineTag(machineId),
+		Tag:      names.MachineTag(machineId),
 		Password: "unimportant",
 		CACert:   []byte(testing.CACert),
 	}
@@ -72,7 +75,7 @@
 func FakeAPIInfo(machineId string) *api.Info {
 	return &api.Info{
 		Addrs:    []string{"0.1.2.3:1234"},
-		Tag:      state.MachineTag(machineId),
+		Tag:      names.MachineTag(machineId),
 		Password: "unimportant",
 		CACert:   []byte(testing.CACert),
 	}

=== added directory 'names'
=== added file 'names/constants.go'
--- names/constants.go	1970-01-01 00:00:00 +0000
+++ names/constants.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,18 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names
+
+const (
+	UnitTagPrefix    = "unit-"
+	MachineTagPrefix = "machine-"
+	ServiceTagPrefix = "service-"
+	EnvironTagPrefix = "environment-"
+	UserTagPrefix    = "user-"
+
+	ServiceSnippet       = "[a-z][a-z0-9]*(-[a-z0-9]*[a-z][a-z0-9]*)*"
+	NumberSnippet        = "(0|[1-9][0-9]*)"
+	ContainerSnippet     = "(/[a-z]+/" + NumberSnippet + ")"
+	MachineSnippet       = NumberSnippet + ContainerSnippet + "*"
+	ContainerSpecSnippet = "(([a-z])*:)?"
+)

=== added file 'names/environ.go'
--- names/environ.go	1970-01-01 00:00:00 +0000
+++ names/environ.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,8 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names
+
+func EnvironTag(name string) string {
+	return EnvironTagPrefix + name
+}

=== added file 'names/machine.go'
--- names/machine.go	1970-01-01 00:00:00 +0000
+++ names/machine.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,44 @@
+package names
+
+import (
+	"fmt"
+	"regexp"
+	"strings"
+)
+
+var (
+	validMachine               = regexp.MustCompile("^" + MachineSnippet + "$")
+	validMachineOrNewContainer = regexp.MustCompile("^" + ContainerSpecSnippet + MachineSnippet + "$")
+)
+
+// IsMachineId returns whether id is a valid machine id.
+func IsMachineId(id string) bool {
+	return validMachine.MatchString(id)
+}
+
+// IsMachineOrNewContainer returns whether spec is a valid machine id
+// or new container definition.
+func IsMachineOrNewContainer(spec string) bool {
+	return validMachineOrNewContainer.MatchString(spec)
+}
+
+// MachineTag returns the tag for the machine wi th the given id.
+func MachineTag(id string) string {
+	tag := fmt.Sprintf("%s%s", MachineTagPrefix, id)
+	// Containers require "/" to be replaced by "-".
+	tag = strings.Replace(tag, "/", "-", -1)
+	return tag
+}
+
+// MachineIdFromTag returns the machine id that was used to create the
+// tag.
+func MachineIdFromTag(tag string) (string, error) {
+	if !strings.HasPrefix(tag, MachineTagPrefix) {
+		return "", fmt.Errorf("invalid machine tag format: %v", tag)
+	}
+	// Strip off the "machine-" prefix.
+	id := tag[len(MachineTagPrefix):]
+	// Put the slashes back.
+	id = strings.Replace(id, "-", "/", -1)
+	return id, nil
+}

=== added file 'names/machine_test.go'
--- names/machine_test.go	1970-01-01 00:00:00 +0000
+++ names/machine_test.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,45 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names_test
+
+import (
+	stdtesting "testing"
+
+	gc "launchpad.net/gocheck"
+
+	"launchpad.net/juju-core/names"
+)
+
+type machineSuite struct{}
+
+var _ = gc.Suite(&unitSuite{})
+
+func Test(t *stdtesting.T) {
+	gc.TestingT(t)
+}
+
+func (s *machineSuite) TestMachineTag(c *gc.C) {
+	c.Assert(names.MachineTag("10"), gc.Equals, "machine-10")
+	// Check a container id.
+	c.Assert(names.MachineTag("10/lxc/1"), gc.Equals, "machine-10-lxc-1")
+}
+
+func (s *machineSuite) TestMachineIdFromTag(c *gc.C) {
+	id, err := names.MachineIdFromTag("machine-10")
+	c.Assert(err, gc.IsNil)
+	c.Assert(id, gc.Equals, "10")
+	// Check a container id.
+	id, err = names.MachineIdFromTag("machine-10-lxc-1")
+	c.Assert(err, gc.IsNil)
+	c.Assert(id, gc.Equals, "10/lxc/1")
+	// Check reversability.
+	nested := "2/kvm/0/lxc/3"
+	id, err = names.MachineIdFromTag(names.MachineTag(nested))
+	c.Assert(err, gc.IsNil)
+	c.Assert(id, gc.Equals, nested)
+	// Try with an invalid tag format.
+	id, err = names.MachineIdFromTag("foo")
+	c.Assert(err, gc.ErrorMatches, "invalid machine tag format: foo")
+	c.Assert(id, gc.Equals, "")
+}

=== added file 'names/service.go'
--- names/service.go	1970-01-01 00:00:00 +0000
+++ names/service.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,20 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names
+
+import (
+	"regexp"
+)
+
+var validService = regexp.MustCompile("^" + ServiceSnippet + "$")
+
+// IsServiceName returns whether name is a valid service name.
+func IsServiceName(name string) bool {
+	return validService.MatchString(name)
+}
+
+// ServiceTag returns the tag for the service with the given name.
+func ServiceTag(serviceName string) string {
+	return ServiceTagPrefix + serviceName
+}

=== added file 'names/unit.go'
--- names/unit.go	1970-01-01 00:00:00 +0000
+++ names/unit.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,34 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names
+
+import (
+	"fmt"
+	"regexp"
+	"strings"
+)
+
+var validUnit = regexp.MustCompile("^" + ServiceSnippet + "/" + NumberSnippet + "$")
+
+// UnitTag returns the tag for the unit with the given name.
+func UnitTag(unitName string) string {
+	return UnitTagPrefix + strings.Replace(unitName, "/", "-", -1)
+}
+
+// UnitNameFromTag returns the unit name that was used to create the tag.
+func UnitNameFromTag(tag string) (string, error) {
+	if !strings.HasPrefix(tag, UnitTagPrefix) {
+		return "", fmt.Errorf("invalid unit tag format: %v", tag)
+	}
+	// Strip off the "unit-" prefix.
+	name := tag[len(UnitTagPrefix):]
+	// Put the slashes back.
+	name = strings.Replace(name, "-", "/", -1)
+	return name, nil
+}
+
+// IsUnitName returns whether name is a valid unit name.
+func IsUnitName(name string) bool {
+	return validUnit.MatchString(name)
+}

=== added file 'names/unit_test.go'
--- names/unit_test.go	1970-01-01 00:00:00 +0000
+++ names/unit_test.go	2013-07-29 15:15:41 +0000
@@ -0,0 +1,29 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package names_test
+
+import (
+	gc "launchpad.net/gocheck"
+
+	"launchpad.net/juju-core/names"
+)
+
+type unitSuite struct{}
+
+var _ = gc.Suite(&unitSuite{})
+
+func (s *unitSuite) TestUnitNameFromTag(c *gc.C) {
+	// Try both valid and invalid tag formats.
+	tag, err := names.UnitNameFromTag("unit-wordpress-0")
+	c.Assert(err, gc.IsNil)
+	c.Assert(tag, gc.Equals, "wordpress/0")
+
+	tag, err = names.UnitNameFromTag("foo")
+	c.Assert(err, gc.ErrorMatches, "invalid unit tag format: foo")
+	c.Assert(tag, gc.Equals, "")
+}
+
+func (s *unitSuite) TestUnitTag(c *gc.C) {
+	c.Assert(names.UnitTag("wordpress/2"), gc.Equals, "unit-wordpress-2")
+}

=== modified file 'state/api/deployer/unit.go'
--- state/api/deployer/unit.go	2013-07-26 15:16:59 +0000
+++ state/api/deployer/unit.go	2013-07-29 15:15:41 +0000
@@ -4,8 +4,7 @@
 package deployer
 
 import (
-	"strings"
-
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 )
 
@@ -21,23 +20,12 @@
 	return u.tag
 }
 
-const unitTagPrefix = "unit-"
-
-// UnitTag returns the tag for the
-// unit with the given name.
-func UnitTag(unitName string) string {
-	return unitTagPrefix + strings.Replace(unitName, "/", "-", -1)
-}
-
 // Name returns the unit's name.
 func (u *Unit) Name() string {
-	if !strings.HasPrefix(u.tag, unitTagPrefix) {
+	name, err := names.UnitNameFromTag(u.tag)
+	if err != nil {
 		return ""
 	}
-	// Strip off the "unit-" prefix.
-	name := u.tag[len(unitTagPrefix):]
-	// Put the slashes back.
-	name = strings.Replace(name, "-", "/", -1)
 	return name
 }
 

=== modified file 'state/apiserver/deployer/deployer.go'
--- state/apiserver/deployer/deployer.go	2013-07-26 15:16:59 +0000
+++ state/apiserver/deployer/deployer.go	2013-07-29 15:15:41 +0000
@@ -7,6 +7,7 @@
 	"fmt"
 
 	"launchpad.net/juju-core/environs"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/state/api/params"
@@ -28,7 +29,11 @@
 // getAllUnits returns a list of all principal and subordinate units
 // assigned to the given machine.
 func getAllUnits(st *state.State, machineTag string) ([]string, error) {
-	machine, err := st.Machine(state.MachineIdFromTag(machineTag))
+	id, err := names.MachineIdFromTag(machineTag)
+	if err != nil {
+		return nil, err
+	}
+	machine, err := st.Machine(id)
 	if err != nil {
 		return nil, err
 	}
@@ -60,7 +65,7 @@
 		// Then we just check if the unit is already known.
 		return func(tag string) bool {
 			for _, unit := range units {
-				if state.UnitTag(unit) == tag {
+				if names.UnitTag(unit) == tag {
 					return true
 				}
 			}
@@ -88,15 +93,19 @@
 		err := common.ErrPerm
 		if d.authorizer.AuthOwner(entity.Tag) {
 			var machine *state.Machine
-			machine, err = d.st.Machine(state.MachineIdFromTag(entity.Tag))
+			var id string
+			id, err = names.MachineIdFromTag(entity.Tag)
 			if err == nil {
-				watch := machine.WatchUnits()
-				// Consume the initial event and forward it to the result.
-				if changes, ok := <-watch.Changes(); ok {
-					result.Results[i].StringsWatcherId = d.resources.Register(watch)
-					result.Results[i].Changes = changes
-				} else {
-					err = watcher.MustErr(watch)
+				machine, err = d.st.Machine(id)
+				if err == nil {
+					watch := machine.WatchUnits()
+					// Consume the initial event and forward it to the result.
+					if changes, ok := <-watch.Changes(); ok {
+						result.Results[i].StringsWatcherId = d.resources.Register(watch)
+						result.Results[i].Changes = changes
+					} else {
+						err = watcher.MustErr(watch)
+					}
 				}
 			}
 		}

=== modified file 'state/apiserver/deployer/deployer_test.go'
--- state/apiserver/deployer/deployer_test.go	2013-07-26 11:24:48 +0000
+++ state/apiserver/deployer/deployer_test.go	2013-07-29 15:15:41 +0000
@@ -11,6 +11,7 @@
 
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/juju/testing"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/apiserver/common"
@@ -87,7 +88,7 @@
 	// Create a FakeAuthorizer so we can check permissions,
 	// set up assuming machine 1 has logged in.
 	s.authorizer = apiservertesting.FakeAuthorizer{
-		Tag:          state.MachineTag(s.machine1.Id()),
+		Tag:          names.MachineTag(s.machine1.Id()),
 		LoggedIn:     true,
 		Manager:      false,
 		MachineAgent: true,

=== modified file 'state/apiserver/machine/agent.go'
--- state/apiserver/machine/agent.go	2013-07-19 15:36:42 +0000
+++ state/apiserver/machine/agent.go	2013-07-29 15:15:41 +0000
@@ -6,6 +6,7 @@
 package machine
 
 import (
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/apiserver/common"
@@ -57,7 +58,11 @@
 		err = common.ErrPerm
 		return
 	}
-	machine, err := api.st.Machine(state.MachineIdFromTag(tag))
+	id, err := names.MachineIdFromTag(tag)
+	if err != nil {
+		return
+	}
+	machine, err := api.st.Machine(id)
 	if err != nil {
 		return
 	}

=== modified file 'state/apiserver/machine/machiner.go'
--- state/apiserver/machine/machiner.go	2013-07-24 17:16:20 +0000
+++ state/apiserver/machine/machiner.go	2013-07-29 15:15:41 +0000
@@ -4,6 +4,7 @@
 package machine
 
 import (
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/apiserver/common"
@@ -49,9 +50,13 @@
 		err := common.ErrPerm
 		if m.auth.AuthOwner(arg.Tag) {
 			var machine *state.Machine
-			machine, err = m.st.Machine(state.MachineIdFromTag(arg.Tag))
+			var id string
+			id, err = names.MachineIdFromTag(arg.Tag)
 			if err == nil {
-				err = machine.SetStatus(arg.Status, arg.Info)
+				machine, err = m.st.Machine(id)
+				if err == nil {
+					err = machine.SetStatus(arg.Status, arg.Info)
+				}
 			}
 		}
 		result.Results[i].Error = common.ServerError(err)
@@ -71,17 +76,21 @@
 		err := common.ErrPerm
 		if m.auth.AuthOwner(entity.Tag) {
 			var machine *state.Machine
-			machine, err = m.st.Machine(state.MachineIdFromTag(entity.Tag))
+			var id string
+			id, err = names.MachineIdFromTag(entity.Tag)
 			if err == nil {
-				watch := machine.Watch()
-				// Consume the initial event. Technically, API
-				// calls to Watch 'transmit' the initial event
-				// in the Watch response. But NotifyWatchers
-				// have no state to transmit.
-				if _, ok := <-watch.Changes(); ok {
-					result.Results[i].NotifyWatcherId = m.resources.Register(watch)
-				} else {
-					err = watcher.MustErr(watch)
+				machine, err = m.st.Machine(id)
+				if err == nil {
+					watch := machine.Watch()
+					// Consume the initial event. Technically, API
+					// calls to Watch 'transmit' the initial event
+					// in the Watch response. But NotifyWatchers
+					// have no state to transmit.
+					if _, ok := <-watch.Changes(); ok {
+						result.Results[i].NotifyWatcherId = m.resources.Register(watch)
+					} else {
+						err = watcher.MustErr(watch)
+					}
 				}
 			}
 		}
@@ -103,9 +112,13 @@
 		err := common.ErrPerm
 		if m.auth.AuthOwner(entity.Tag) {
 			var machine *state.Machine
-			machine, err = m.st.Machine(state.MachineIdFromTag(entity.Tag))
+			var id string
+			id, err = names.MachineIdFromTag(entity.Tag)
 			if err == nil {
-				err = machine.EnsureDead()
+				machine, err = m.st.Machine(id)
+				if err == nil {
+					err = machine.EnsureDead()
+				}
 			}
 		}
 		result.Results[i].Error = common.ServerError(err)

=== modified file 'state/apiserver/upgrader/upgrader.go'
--- state/apiserver/upgrader/upgrader.go	2013-07-25 14:12:12 +0000
+++ state/apiserver/upgrader/upgrader.go	2013-07-29 15:15:41 +0000
@@ -8,6 +8,7 @@
 
 	"launchpad.net/juju-core/agent/tools"
 	"launchpad.net/juju-core/environs"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/apiserver/common"
@@ -65,7 +66,11 @@
 	if !u.authorizer.AuthOwner(entity.Tag) {
 		return nil, common.ErrPerm
 	}
-	machine, err := u.st.Machine(state.MachineIdFromTag(entity.Tag))
+	id, err := names.MachineIdFromTag(entity.Tag)
+	if err != nil {
+		return nil, err
+	}
+	machine, err := u.st.Machine(id)
 	if err != nil {
 		return nil, err
 	}

=== modified file 'state/environ.go'
--- state/environ.go	2013-07-09 10:32:23 +0000
+++ state/environ.go	2013-07-29 15:15:41 +0000
@@ -8,6 +8,7 @@
 	"labix.org/v2/mgo/txn"
 
 	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/names"
 )
 
 // environGlobalKey is the key for the environment, its
@@ -52,7 +53,7 @@
 // The returned name will be different from other Tag values returned
 // by any other entities from the same state.
 func (e Environment) Tag() string {
-	return "environment-" + e.doc.Name
+	return names.EnvironTag(e.doc.Name)
 }
 
 // UUID returns the universally unique identifier of the environment.

=== modified file 'state/machine.go'
--- state/machine.go	2013-07-24 16:08:50 +0000
+++ state/machine.go	2013-07-29 15:15:41 +0000
@@ -15,6 +15,7 @@
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/presence"
 	"launchpad.net/juju-core/utils"
@@ -148,36 +149,11 @@
 	return instData, nil
 }
 
-const machineTagPrefix = "machine-"
-
-// MachineTag returns the tag for the
-// machine with the given id.
-func MachineTag(id string) string {
-	tag := fmt.Sprintf("%s%s", machineTagPrefix, id)
-	// Containers require "/" to be replaced by "-".
-	tag = strings.Replace(tag, "/", "-", -1)
-	return tag
-}
-
-// MachineIdFromTag returns the machine id that was used to create the tag.
-func MachineIdFromTag(tag string) string {
-	// TODO(dimitern): Possibly change this to return (string, error),
-	// so the case below can be reported.
-	if !strings.HasPrefix(tag, machineTagPrefix) {
-		return ""
-	}
-	// Strip off the "machine-" prefix.
-	id := tag[len(machineTagPrefix):]
-	// Put the slashes back.
-	id = strings.Replace(id, "-", "/", -1)
-	return id
-}
-
 // Tag returns a name identifying the machine that is safe to use
 // as a file name.  The returned name will be different from other
 // Tag values returned by any other entities from the same state.
 func (m *Machine) Tag() string {
-	return MachineTag(m.Id())
+	return names.MachineTag(m.Id())
 }
 
 // Life returns whether the machine is Alive, Dying or Dead.

=== modified file 'state/machine_test.go'
--- state/machine_test.go	2013-07-24 16:08:50 +0000
+++ state/machine_test.go	2013-07-29 15:15:41 +0000
@@ -243,23 +243,6 @@
 	c.Assert(s.machine.Tag(), Equals, "machine-0")
 }
 
-func (s *MachineSuite) TestMachineTag(c *C) {
-	c.Assert(state.MachineTag("10"), Equals, "machine-10")
-	// Check a container id.
-	c.Assert(state.MachineTag("10/lxc/1"), Equals, "machine-10-lxc-1")
-}
-
-func (s *MachineSuite) TestMachineIdFromTag(c *C) {
-	c.Assert(state.MachineIdFromTag("machine-10"), Equals, "10")
-	// Check a container id.
-	c.Assert(state.MachineIdFromTag("machine-10-lxc-1"), Equals, "10/lxc/1")
-	// Check reversability.
-	nested := "2/kvm/0/lxc/3"
-	c.Assert(state.MachineIdFromTag(state.MachineTag(nested)), Equals, nested)
-	// Try with an invalid tag format.
-	c.Assert(state.MachineIdFromTag("foo"), Equals, "")
-}
-
 func (s *MachineSuite) TestSetMongoPassword(c *C) {
 	testSetMongoPassword(c, func(st *state.State) (entity, error) {
 		return st.Machine(s.machine.Id())

=== modified file 'state/relationunit.go'
--- state/relationunit.go	2013-07-10 21:30:08 +0000
+++ state/relationunit.go	2013-07-29 15:15:41 +0000
@@ -13,6 +13,7 @@
 
 	"launchpad.net/juju-core/charm"
 	errors "launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/utils"
 )
 
@@ -331,7 +332,7 @@
 // of the lifetime of the unit.
 func (ru *RelationUnit) ReadSettings(uname string) (m map[string]interface{}, err error) {
 	defer utils.ErrorContextf(&err, "cannot read settings for unit %q in relation %q", uname, ru.relation)
-	if !IsUnitName(uname) {
+	if !names.IsUnitName(uname) {
 		return nil, fmt.Errorf("%q is not a valid unit name", uname)
 	}
 	key, err := ru.key(uname)

=== modified file 'state/service.go'
--- state/service.go	2013-07-10 21:30:08 +0000
+++ state/service.go	2013-07-29 15:15:41 +0000
@@ -16,6 +16,7 @@
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/utils"
 )
@@ -66,7 +67,7 @@
 // as a file name.  The returned name will be different from other
 // Tag values returned by any other entities from the same state.
 func (s *Service) Tag() string {
-	return "service-" + s.Name()
+	return names.ServiceTag(s.Name())
 }
 
 // serviceGlobalKey returns the global database key for the service
@@ -691,7 +692,7 @@
 
 // Unit returns the service's unit with name.
 func (s *Service) Unit(name string) (*Unit, error) {
-	if !IsUnitName(name) {
+	if !names.IsUnitName(name) {
 		return nil, fmt.Errorf("%q is not a valid unit name", name)
 	}
 	udoc := &unitDoc{}

=== modified file 'state/state.go'
--- state/state.go	2013-07-25 11:14:42 +0000
+++ state/state.go	2013-07-29 15:15:41 +0000
@@ -9,7 +9,6 @@
 import (
 	"fmt"
 	"net/url"
-	"regexp"
 	"sort"
 	"strconv"
 	"strings"
@@ -26,6 +25,7 @@
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/multiwatcher"
 	"launchpad.net/juju-core/state/presence"
@@ -36,42 +36,9 @@
 // TODO(niemeyer): This must not be exported.
 type D []bson.DocElem
 
-const serviceSnippet = "[a-z][a-z0-9]*(-[a-z0-9]*[a-z][a-z0-9]*)*"
-const numberSnippet = "(0|[1-9][0-9]*)"
-const containerSnippet = "(/[a-z]+/" + numberSnippet + ")"
-const machineSnippet = numberSnippet + containerSnippet + "*"
-const containerSpecSnippet = "(([a-z])*:)?"
-
-var (
-	validService               = regexp.MustCompile("^" + serviceSnippet + "$")
-	validUnit                  = regexp.MustCompile("^" + serviceSnippet + "/" + numberSnippet + "$")
-	validMachine               = regexp.MustCompile("^" + machineSnippet + "$")
-	validMachineOrNewContainer = regexp.MustCompile("^" + containerSpecSnippet + machineSnippet + "$")
-)
-
 // BootstrapNonce is used as a nonce for the state server machine.
 const BootstrapNonce = "user-admin:bootstrap"
 
-// IsServiceName returns whether name is a valid service name.
-func IsServiceName(name string) bool {
-	return validService.MatchString(name)
-}
-
-// IsUnitName returns whether name is a valid unit name.
-func IsUnitName(name string) bool {
-	return validUnit.MatchString(name)
-}
-
-// IsMachineId returns whether id is a valid machine id.
-func IsMachineId(id string) bool {
-	return validMachine.MatchString(id)
-}
-
-// IsMachineOrNewContainer returns whether spec is a valid machine id or new container definition.
-func IsMachineOrNewContainer(spec string) bool {
-	return validMachineOrNewContainer.MatchString(spec)
-}
-
 // State represents the state of an environment
 // managed by juju.
 type State struct {
@@ -603,31 +570,35 @@
 		return nil, fmt.Errorf("invalid entity tag %q", tag)
 	}
 	prefix, id := tag[0:i], tag[i+1:]
+	prefix += "-"
 	switch prefix {
-	case "machine":
-		id = MachineIdFromTag(tag)
-		if !IsMachineId(id) {
+	case names.MachineTagPrefix:
+		id, err := names.MachineIdFromTag(tag)
+		if err != nil {
+			return nil, err
+		}
+		if !names.IsMachineId(id) {
 			return nil, fmt.Errorf("invalid entity tag %q", tag)
 		}
 		return st.Machine(id)
-	case "unit":
+	case names.UnitTagPrefix:
 		i := strings.LastIndex(id, "-")
 		if i == -1 {
 			return nil, fmt.Errorf("invalid entity tag %q", tag)
 		}
 		name := id[:i] + "/" + id[i+1:]
-		if !IsUnitName(name) {
+		if !names.IsUnitName(name) {
 			return nil, fmt.Errorf("invalid entity tag %q", tag)
 		}
 		return st.Unit(name)
-	case "user":
+	case names.UserTagPrefix:
 		return st.User(id)
-	case "service":
-		if !IsServiceName(id) {
+	case names.ServiceTagPrefix:
+		if !names.IsServiceName(id) {
 			return nil, fmt.Errorf("invalid entity tag %q", tag)
 		}
 		return st.Service(id)
-	case "environment":
+	case names.EnvironTagPrefix:
 		conf, err := st.EnvironConfig()
 		if err != nil {
 			return nil, err
@@ -651,12 +622,13 @@
 	}
 	id := parts[1]
 	var coll string
-	switch parts[0] {
-	case "machine":
+	tagPrefix := parts[0] + "-"
+	switch tagPrefix {
+	case names.MachineTagPrefix:
 		coll = st.machines.Name
-	case "service":
+	case names.ServiceTagPrefix:
 		coll = st.services.Name
-	case "unit":
+	case names.UnitTagPrefix:
 		coll = st.units.Name
 		// Handle replacements occurring when an entity name is created
 		// for a unit.
@@ -665,7 +637,7 @@
 			return "", "", fmt.Errorf("invalid entity name %q", tag)
 		}
 		id = id[:idx] + "/" + id[idx+1:]
-	case "user":
+	case names.UserTagPrefix:
 		coll = st.users.Name
 	default:
 		return "", "", fmt.Errorf("invalid entity name %q", tag)
@@ -743,7 +715,7 @@
 func (st *State) AddService(name string, ch *Charm) (service *Service, err error) {
 	defer utils.ErrorContextf(&err, "cannot add service %q", name)
 	// Sanity checks.
-	if !IsServiceName(name) {
+	if !names.IsServiceName(name) {
 		return nil, fmt.Errorf("invalid name")
 	}
 	if ch == nil {
@@ -803,7 +775,7 @@
 
 // Service returns a service state by name.
 func (st *State) Service(name string) (service *Service, err error) {
-	if !IsServiceName(name) {
+	if !names.IsServiceName(name) {
 		return nil, fmt.Errorf("%q is not a valid service name", name)
 	}
 	sdoc := &serviceDoc{}
@@ -1075,7 +1047,7 @@
 
 // Unit returns a unit by name.
 func (st *State) Unit(name string) (*Unit, error) {
-	if !IsUnitName(name) {
+	if !names.IsUnitName(name) {
 		return nil, fmt.Errorf("%q is not a valid unit name", name)
 	}
 	doc := unitDoc{}
@@ -1332,10 +1304,10 @@
 }
 
 var tagPrefix = map[byte]string{
-	'm': machineTagPrefix,
-	's': "service-",
-	'u': unitTagPrefix,
-	'e': "environment-",
+	'm': names.MachineTagPrefix,
+	's': names.ServiceTagPrefix,
+	'u': names.UnitTagPrefix,
+	'e': names.EnvironTagPrefix,
 }
 
 func tagForGlobalKey(key string) (string, bool) {

=== modified file 'state/state_test.go'
--- state/state_test.go	2013-07-24 16:08:50 +0000
+++ state/state_test.go	2013-07-29 15:15:41 +0000
@@ -19,6 +19,7 @@
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	statetesting "launchpad.net/juju-core/state/testing"
@@ -1047,15 +1048,15 @@
 
 func (*StateSuite) TestNameChecks(c *gc.C) {
 	assertService := func(s string, expect bool) {
-		c.Assert(state.IsServiceName(s), gc.Equals, expect)
+		c.Assert(names.IsServiceName(s), gc.Equals, expect)
 		// Check that anything that is considered a valid service name
 		// is also (in)valid if a(n) (in)valid unit designator is added
 		// to it.
-		c.Assert(state.IsUnitName(s+"/0"), gc.Equals, expect)
-		c.Assert(state.IsUnitName(s+"/99"), gc.Equals, expect)
-		c.Assert(state.IsUnitName(s+"/-1"), gc.Equals, false)
-		c.Assert(state.IsUnitName(s+"/blah"), gc.Equals, false)
-		c.Assert(state.IsUnitName(s+"/"), gc.Equals, false)
+		c.Assert(names.IsUnitName(s+"/0"), gc.Equals, expect)
+		c.Assert(names.IsUnitName(s+"/99"), gc.Equals, expect)
+		c.Assert(names.IsUnitName(s+"/-1"), gc.Equals, false)
+		c.Assert(names.IsUnitName(s+"/blah"), gc.Equals, false)
+		c.Assert(names.IsUnitName(s+"/"), gc.Equals, false)
 	}
 	// Service names must be non-empty...
 	assertService("", false)
@@ -1079,7 +1080,7 @@
 	assertService("foo-2", false)
 
 	assertMachine := func(s string, expect bool) {
-		c.Assert(state.IsMachineId(s), gc.Equals, expect)
+		c.Assert(names.IsMachineId(s), gc.Equals, expect)
 	}
 	assertMachine("0", true)
 	assertMachine("00", false)
@@ -1104,7 +1105,7 @@
 	assertMachine("0/lxc/1/embedded/2", true)
 
 	assertMachineOrNewContainer := func(s string, expect bool) {
-		c.Assert(state.IsMachineOrNewContainer(s), gc.Equals, expect)
+		c.Assert(names.IsMachineOrNewContainer(s), gc.Equals, expect)
 	}
 	assertMachineOrNewContainer("0", true)
 	assertMachineOrNewContainer("00", false)

=== modified file 'state/unit.go'
--- state/unit.go	2013-07-26 11:24:48 +0000
+++ state/unit.go	2013-07-29 15:15:41 +0000
@@ -7,7 +7,6 @@
 	stderrors "errors"
 	"fmt"
 	"sort"
-	"strings"
 	"time"
 
 	"labix.org/v2/mgo"
@@ -19,6 +18,7 @@
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/presence"
 	"launchpad.net/juju-core/utils"
@@ -433,9 +433,9 @@
 // the unit. If no such entity can be determined, false is returned.
 func (u *Unit) DeployerTag() (string, bool) {
 	if u.doc.Principal != "" {
-		return UnitTag(u.doc.Principal), true
+		return names.UnitTag(u.doc.Principal), true
 	} else if u.doc.MachineId != "" {
-		return MachineTag(u.doc.MachineId), true
+		return names.MachineTag(u.doc.MachineId), true
 	}
 	return "", false
 }
@@ -628,33 +628,11 @@
 	return u.st.pwatcher.Alive(u.globalKey())
 }
 
-const unitTagPrefix = "unit-"
-
-// UnitTag returns the tag for the
-// unit with the given name.
-func UnitTag(unitName string) string {
-	return unitTagPrefix + strings.Replace(unitName, "/", "-", -1)
-}
-
-// UnitNameFromTag returns the unit name that was used to create the tag.
-func UnitNameFromTag(tag string) string {
-	// TODO(dimitern): Possibly change this to return (string, error),
-	// so the case below can be reported.
-	if !strings.HasPrefix(tag, unitTagPrefix) {
-		return ""
-	}
-	// Strip off the "unit-" prefix.
-	name := tag[len(unitTagPrefix):]
-	// Put the slashes back.
-	name = strings.Replace(name, "-", "/", -1)
-	return name
-}
-
 // Tag returns a name identifying the unit that is safe to use
 // as a file name.  The returned name will be different from other
 // Tag values returned by any other entities from the same state.
 func (u *Unit) Tag() string {
-	return UnitTag(u.Name())
+	return names.UnitTag(u.Name())
 }
 
 // WaitAgentAlive blocks until the respective agent is alive.

=== modified file 'state/unit_test.go'
--- state/unit_test.go	2013-07-19 09:53:58 +0000
+++ state/unit_test.go	2013-07-29 15:15:41 +0000
@@ -44,12 +44,6 @@
 	c.Assert(err, checkers.Satisfies, errors.IsNotFoundError)
 }
 
-func (s *UnitSuite) TestUnitNameFromTag(c *C) {
-	// Try both valid and invalid tag formats.
-	c.Assert(state.UnitNameFromTag("unit-wordpress-0"), Equals, "wordpress/0")
-	c.Assert(state.UnitNameFromTag("foo"), Equals, "")
-}
-
 func (s *UnitSuite) TestService(c *C) {
 	svc, err := s.unit.Service()
 	c.Assert(err, IsNil)
@@ -533,10 +527,6 @@
 	c.Assert(s.unit.Tag(), Equals, "unit-wordpress-0")
 }
 
-func (s *UnitSuite) TestUnitTag(c *C) {
-	c.Assert(state.UnitTag("wordpress/2"), Equals, "unit-wordpress-2")
-}
-
 func (s *UnitSuite) TestSetMongoPassword(c *C) {
 	testSetMongoPassword(c, func(st *state.State) (entity, error) {
 		return st.Unit(s.unit.Name())

=== modified file 'state/watcher.go'
--- state/watcher.go	2013-07-10 17:23:31 +0000
+++ state/watcher.go	2013-07-29 15:15:41 +0000
@@ -16,6 +16,7 @@
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/watcher"
 	"launchpad.net/juju-core/utils/set"
 )
@@ -158,7 +159,7 @@
 // lifecycles of containers on a machine.
 func (m *Machine) WatchContainers(ctype instance.ContainerType) StringsWatcher {
 	members := D{{"parent", m.doc.Id}}
-	match := fmt.Sprintf("^%s/%s/%s$", m.doc.Id, ctype, numberSnippet)
+	match := fmt.Sprintf("^%s/%s/%s$", m.doc.Id, ctype, names.NumberSnippet)
 	child := regexp.MustCompile(match)
 	filter := func(key interface{}) bool {
 		return child.MatchString(key.(string))

=== modified file 'worker/deployer/deployer.go'
--- worker/deployer/deployer.go	2013-07-26 11:24:48 +0000
+++ worker/deployer/deployer.go	2013-07-29 15:15:41 +0000
@@ -10,6 +10,7 @@
 	"launchpad.net/tomb"
 
 	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/names"
 	apideployer "launchpad.net/juju-core/state/api/deployer"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/watcher"
@@ -88,7 +89,7 @@
 // changed ensures that the named unit is deployed, recalled, or removed, as
 // indicated by its state.
 func (d *Deployer) changed(unitName string) error {
-	unitTag := apideployer.UnitTag(unitName)
+	unitTag := names.UnitTag(unitName)
 	// Determine unit life state, and whether we're responsible for it.
 	logger.Infof("checking unit %q", unitName)
 	var life params.Life

=== modified file 'worker/deployer/deployer_test.go'
--- worker/deployer/deployer_test.go	2013-07-26 15:16:59 +0000
+++ worker/deployer/deployer_test.go	2013-07-29 15:15:41 +0000
@@ -13,6 +13,7 @@
 
 	"launchpad.net/juju-core/errors"
 	jujutesting "launchpad.net/juju-core/juju/testing"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	apideployer "launchpad.net/juju-core/state/api/deployer"
@@ -182,7 +183,7 @@
 	ctx := s.getContext(c)
 	machineId, err := u.AssignedMachineId()
 	c.Assert(err, IsNil)
-	dep := deployer.NewDeployer(s.deployerState, ctx, state.MachineTag(machineId))
+	dep := deployer.NewDeployer(s.deployerState, ctx, names.MachineTag(machineId))
 	defer stop(c, dep)
 
 	// Add a subordinate, and wait for it to be deployed.
@@ -235,7 +236,7 @@
 	ctx := s.getContext(c)
 	machineId, err := u.AssignedMachineId()
 	c.Assert(err, IsNil)
-	dep := deployer.NewDeployer(s.deployerState, ctx, state.MachineTag(machineId))
+	dep := deployer.NewDeployer(s.deployerState, ctx, names.MachineTag(machineId))
 	defer stop(c, dep)
 	s.waitFor(c, isRemoved(s.State, sub0.Name()))
 	s.waitFor(c, isRemoved(s.State, sub1.Name()))

=== modified file 'worker/deployer/simple.go'
--- worker/deployer/simple.go	2013-07-26 15:16:59 +0000
+++ worker/deployer/simple.go	2013-07-29 15:15:41 +0000
@@ -14,9 +14,9 @@
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/agent/tools"
 	"launchpad.net/juju-core/log/syslog"
-	"launchpad.net/juju-core/state" // Only because of state.Info and IsUnitName
+	"launchpad.net/juju-core/names"
+	"launchpad.net/juju-core/state" // Only because of state.Info
 	"launchpad.net/juju-core/state/api"
-	apideployer "launchpad.net/juju-core/state/api/deployer"
 	"launchpad.net/juju-core/upstart"
 	"launchpad.net/juju-core/version"
 )
@@ -82,7 +82,7 @@
 	}
 
 	// Link the current tools for use by the new agent.
-	tag := apideployer.UnitTag(unitName)
+	tag := names.UnitTag(unitName)
 	_, err = tools.ChangeAgentTools(ctx.dataDir, tag, version.Current)
 	toolsDir := tools.ToolsDir(ctx.dataDir, tag)
 	defer removeOnErr(&err, toolsDir)
@@ -179,7 +179,7 @@
 	if err := svc.Remove(); err != nil {
 		return err
 	}
-	tag := apideployer.UnitTag(unitName)
+	tag := names.UnitTag(unitName)
 	agentDir := tools.Dir(ctx.dataDir, tag)
 	if err := os.RemoveAll(agentDir); err != nil {
 		return err
@@ -208,7 +208,7 @@
 	for _, fi := range fis {
 		if groups := deployedRe.FindStringSubmatch(fi.Name()); len(groups) == 4 {
 			unitName := groups[2] + "/" + groups[3]
-			if !state.IsUnitName(unitName) {
+			if !names.IsUnitName(unitName) {
 				continue
 			}
 			installed[unitName] = groups[1]
@@ -232,7 +232,7 @@
 // upstartService returns an upstart.Service corresponding to the specified
 // unit.
 func (ctx *SimpleContext) upstartService(unitName string) *upstart.Service {
-	tag := apideployer.UnitTag(unitName)
+	tag := names.UnitTag(unitName)
 	svcName := "jujud-" + tag
 	svc := upstart.NewService(svcName)
 	svc.InitDir = ctx.initDir

=== modified file 'worker/deployer/simple_test.go'
--- worker/deployer/simple_test.go	2013-07-22 03:05:03 +0000
+++ worker/deployer/simple_test.go	2013-07-29 15:15:41 +0000
@@ -16,6 +16,7 @@
 
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/agent/tools"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api"
 	"launchpad.net/juju-core/testing/checkers"
@@ -208,7 +209,7 @@
 `
 
 func (fix *SimpleToolsFixture) checkUnitInstalled(c *C, name, password string) {
-	tag := state.UnitTag(name)
+	tag := names.UnitTag(name)
 	uconfPath, _, toolsDir, syslogConfPath := fix.paths(tag)
 	uconfData, err := ioutil.ReadFile(uconfPath)
 	c.Assert(err, IsNil)
@@ -268,7 +269,7 @@
 }
 
 func (fix *SimpleToolsFixture) checkUnitRemoved(c *C, name string) {
-	tag := state.UnitTag(name)
+	tag := names.UnitTag(name)
 	confPath, agentDir, toolsDir, syslogConfPath := fix.paths(tag)
 	for _, path := range []string{confPath, agentDir, toolsDir, syslogConfPath} {
 		_, err := ioutil.ReadFile(path)

=== modified file 'worker/provisioner/provisioner_task.go'
--- worker/provisioner/provisioner_task.go	2013-06-30 23:37:23 +0000
+++ worker/provisioner/provisioner_task.go	2013-07-29 15:15:41 +0000
@@ -6,14 +6,16 @@
 import (
 	"fmt"
 
+	"launchpad.net/tomb"
+
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/watcher"
 	"launchpad.net/juju-core/utils"
 	"launchpad.net/juju-core/worker"
-	"launchpad.net/tomb"
 )
 
 type ProvisionerTask interface {
@@ -329,7 +331,7 @@
 	// Generated nonce has the format: "machine-#:UUID". The first
 	// part is a badge, specifying the tag of the machine the provisioner
 	// is running on, while the second part is a random UUID.
-	nonce := fmt.Sprintf("%s:%s", state.MachineTag(task.machineId), uuid.String())
+	nonce := fmt.Sprintf("%s:%s", names.MachineTag(task.machineId), uuid.String())
 	inst, metadata, err := task.broker.StartInstance(machine.Id(), nonce, machine.Series(), cons, stateInfo, apiInfo)
 	if err != nil {
 		// Set the state to error, so the machine will be skipped next

=== modified file 'worker/provisioner/provisioner_test.go'
--- worker/provisioner/provisioner_test.go	2013-07-22 10:16:35 +0000
+++ worker/provisioner/provisioner_test.go	2013-07-29 15:15:41 +0000
@@ -11,12 +11,14 @@
 
 	"labix.org/v2/mgo/bson"
 	. "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/environs/dummy"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/juju/testing"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
 	coretesting "launchpad.net/juju-core/testing"
@@ -130,7 +132,7 @@
 				c.Assert(o.MachineId, Equals, m.Id())
 				nonceParts := strings.SplitN(o.MachineNonce, ":", 2)
 				c.Assert(nonceParts, HasLen, 2)
-				c.Assert(nonceParts[0], Equals, state.MachineTag("0"))
+				c.Assert(nonceParts[0], Equals, names.MachineTag("0"))
 				c.Assert(nonceParts[1], checkers.Satisfies, utils.IsValidUUIDString)
 				c.Assert(o.Secret, Equals, secret)
 				c.Assert(o.Constraints, DeepEquals, cons)

